# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_clientserver_ucm.proto
# plugin: python-betterproto
# Last updated 09/09/2021

from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class CMsgClientUcmAddScreenshot(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    filename: str = betterproto.string_field(2)
    thumbname: str = betterproto.string_field(3)
    vr_filename: str = betterproto.string_field(14)
    rtime32_created: int = betterproto.fixed32_field(4)
    width: int = betterproto.uint32_field(5)
    height: int = betterproto.uint32_field(6)
    permissions: int = betterproto.uint32_field(7)
    caption: str = betterproto.string_field(8)
    shortcut_name: str = betterproto.string_field(9)
    tag: List["CMsgClientUcmAddScreenshotTag"] = betterproto.message_field(10)
    tagged_steamid: List[int] = betterproto.fixed64_field(11)
    spoiler_tag: bool = betterproto.bool_field(12)
    tagged_publishedfileid: List[int] = betterproto.uint64_field(13)


@dataclass(eq=False, repr=False)
class CMsgClientUcmAddScreenshotTag(betterproto.Message):
    tag_name: str = betterproto.string_field(1)
    tag_value: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUcmAddScreenshotResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    screenshotid: int = betterproto.fixed64_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUcmDeleteScreenshot(betterproto.Message):
    screenshotid: int = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientUcmDeleteScreenshotResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientUcmPublishFile(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    file_name: str = betterproto.string_field(2)
    preview_file_name: str = betterproto.string_field(3)
    consumer_app_id: int = betterproto.uint32_field(4)
    title: str = betterproto.string_field(5)
    description: str = betterproto.string_field(6)
    tags: List[str] = betterproto.string_field(8)
    workshop_file: bool = betterproto.bool_field(9)
    visibility: int = betterproto.int32_field(10)
    file_type: int = betterproto.uint32_field(11)
    url: str = betterproto.string_field(12)
    video_provider: int = betterproto.uint32_field(13)
    video_account_name: str = betterproto.string_field(14)
    video_identifier: str = betterproto.string_field(15)
    in_progress: bool = betterproto.bool_field(16)


@dataclass(eq=False, repr=False)
class CMsgClientUcmPublishFileResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    published_file_id: int = betterproto.fixed64_field(2)
    needs_workshop_legal_agreement_acceptance: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientUcmUpdatePublishedFile(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    published_file_id: int = betterproto.fixed64_field(2)
    file_name: str = betterproto.string_field(3)
    preview_file_name: str = betterproto.string_field(4)
    title: str = betterproto.string_field(5)
    description: str = betterproto.string_field(6)
    tags: List[str] = betterproto.string_field(7)
    visibility: int = betterproto.int32_field(8)
    update_file: bool = betterproto.bool_field(9)
    update_preview_file: bool = betterproto.bool_field(10)
    update_title: bool = betterproto.bool_field(11)
    update_description: bool = betterproto.bool_field(12)
    update_tags: bool = betterproto.bool_field(13)
    update_visibility: bool = betterproto.bool_field(14)
    change_description: str = betterproto.string_field(15)
    update_url: bool = betterproto.bool_field(16)
    url: str = betterproto.string_field(17)
    update_content_manifest: bool = betterproto.bool_field(18)
    content_manifest: int = betterproto.fixed64_field(19)
    metadata: str = betterproto.string_field(20)
    update_metadata: bool = betterproto.bool_field(21)
    language: int = betterproto.int32_field(22)
    removed_kvtags: List[str] = betterproto.string_field(23)
    kvtags: List["CMsgClientUcmUpdatePublishedFileKeyValueTag"] = betterproto.message_field(24)
    previews: List["CMsgClientUcmUpdatePublishedFileAdditionalPreview"] = betterproto.message_field(25)
    previews_to_remove: List[int] = betterproto.int32_field(26)
    clear_in_progress: bool = betterproto.bool_field(27)
    remove_all_kvtags: bool = betterproto.bool_field(28)


@dataclass(eq=False, repr=False)
class CMsgClientUcmUpdatePublishedFileKeyValueTag(betterproto.Message):
    key: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUcmUpdatePublishedFileAdditionalPreview(betterproto.Message):
    original_file_name: str = betterproto.string_field(1)
    internal_file_name: str = betterproto.string_field(2)
    videoid: str = betterproto.string_field(3)
    preview_type: int = betterproto.uint32_field(4)
    update_index: int = betterproto.int32_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientUcmUpdatePublishedFileResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    needs_workshop_legal_agreement_acceptance: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUcmDeletePublishedFile(betterproto.Message):
    published_file_id: int = betterproto.fixed64_field(1)
    app_id: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientUcmDeletePublishedFileResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientUcmEnumerateUserSubscribedFilesWithUpdates(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    start_index: int = betterproto.uint32_field(2)
    start_time: int = betterproto.fixed32_field(3)
    desired_revision: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class CMsgClientUcmEnumerateUserSubscribedFilesWithUpdatesResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    subscribed_files: List[
        "CMsgClientUcmEnumerateUserSubscribedFilesWithUpdatesResponsePublishedFileId"
    ] = betterproto.message_field(2)
    total_results: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientUcmEnumerateUserSubscribedFilesWithUpdatesResponsePublishedFileId(betterproto.Message):
    published_file_id: int = betterproto.fixed64_field(1)
    rtime32_subscribed: int = betterproto.fixed32_field(2)
    appid: int = betterproto.uint32_field(3)
    file_hcontent: int = betterproto.fixed64_field(4)
    file_size: int = betterproto.uint32_field(5)
    rtime32_last_updated: int = betterproto.fixed32_field(6)
    is_depot_content: bool = betterproto.bool_field(7)


@dataclass(eq=False, repr=False)
class CMsgClientUcmPublishedFileUpdated(betterproto.Message):
    published_file_id: int = betterproto.fixed64_field(1)
    app_id: int = betterproto.uint32_field(2)
    time_updated: int = betterproto.uint32_field(3)
    hcontent: int = betterproto.fixed64_field(4)
    file_size: int = betterproto.fixed32_field(5)
    is_depot_content: bool = betterproto.bool_field(6)
    revision: int = betterproto.uint32_field(7)


@dataclass(eq=False, repr=False)
class CMsgClientWorkshopItemChangesRequest(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    last_time_updated: int = betterproto.uint32_field(2)
    num_items_needed: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientWorkshopItemChangesResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    update_time: int = betterproto.uint32_field(2)
    workshop_items: List["CMsgClientWorkshopItemChangesResponseWorkshopItemInfo"] = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class CMsgClientWorkshopItemChangesResponseWorkshopItemInfo(betterproto.Message):
    published_file_id: int = betterproto.fixed64_field(1)
    time_updated: int = betterproto.uint32_field(2)
    manifest_id: int = betterproto.fixed64_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientUcmSetUserPublishedFileAction(betterproto.Message):
    published_file_id: int = betterproto.fixed64_field(1)
    app_id: int = betterproto.uint32_field(2)
    action: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientUcmSetUserPublishedFileActionResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class CMsgClientUcmEnumeratePublishedFilesByUserAction(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    start_index: int = betterproto.uint32_field(2)
    action: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientUcmEnumeratePublishedFilesByUserActionResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    published_files: List[
        "CMsgClientUcmEnumeratePublishedFilesByUserActionResponsePublishedFileId"
    ] = betterproto.message_field(2)
    total_results: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CMsgClientUcmEnumeratePublishedFilesByUserActionResponsePublishedFileId(betterproto.Message):
    published_file_id: int = betterproto.fixed64_field(1)
    rtime_time_stamp: int = betterproto.fixed32_field(2)


@dataclass(eq=False, repr=False)
class CMsgClientScreenshotsChanged(betterproto.Message):
    pass
