# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_contentsystem.steamclient.proto
# plugin: python-betterproto
# Last updated 09/09/2021

from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class GetServersForSteamPipeRequest(betterproto.Message):
    cell_id: int = betterproto.uint32_field(1)
    max_servers: int = betterproto.uint32_field(2)
    ip_override: str = betterproto.string_field(3)
    launcher_type: int = betterproto.int32_field(4)
    ipv6_public: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class ServerInfo(betterproto.Message):
    type: str = betterproto.string_field(1)
    source_id: int = betterproto.int32_field(2)
    cell_id: int = betterproto.int32_field(3)
    load: int = betterproto.int32_field(4)
    weighted_load: float = betterproto.float_field(5)
    num_entries_in_client_list: int = betterproto.int32_field(6)
    steam_china_only: bool = betterproto.bool_field(7)
    host: str = betterproto.string_field(8)
    vhost: str = betterproto.string_field(9)
    use_as_proxy: bool = betterproto.bool_field(10)
    proxy_request_path_template: str = betterproto.string_field(11)
    https_support: str = betterproto.string_field(12)
    allowed_app_ids: List[int] = betterproto.uint32_field(13)
    preferred_server: bool = betterproto.bool_field(14)


@dataclass(eq=False, repr=False)
class GetServersForSteamPipeResponse(betterproto.Message):
    servers: List["ServerInfo"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class GetDepotPatchInfoRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    depotid: int = betterproto.uint32_field(2)
    source_manifestid: int = betterproto.uint64_field(3)
    target_manifestid: int = betterproto.uint64_field(4)


@dataclass(eq=False, repr=False)
class GetDepotPatchInfoResponse(betterproto.Message):
    is_available: bool = betterproto.bool_field(1)
    patch_size: int = betterproto.uint64_field(2)
    patched_chunks_size: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class GetClientUpdateHostsRequest(betterproto.Message):
    cached_signature: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class GetClientUpdateHostsResponse(betterproto.Message):
    hosts_kv: str = betterproto.string_field(1)
    valid_until_time: int = betterproto.uint64_field(2)
    ip_country: str = betterproto.string_field(3)
